package ch.uzh.ifi.ddis.ifp.esper.cassandra;

/*
 * #%L
 * Cassandra for Esper
 * %%
 * Copyright (C) 2013 University of Zurich
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * #L%
 */

import java.util.HashMap;
import java.util.Map;

import com.datastax.driver.core.ColumnDefinitions;
import com.datastax.driver.core.DataType;
import com.datastax.driver.core.Row;

public class CassandraRowVisitor {

	private final String[] _columnNames;
	private final DataType.Name[] _columnTypes;

	private final int _size;

	public CassandraRowVisitor(ColumnDefinitions columnDefinitions) {
		_size = columnDefinitions.size();
		_columnNames = new String[_size];
		_columnTypes = new DataType.Name[_size];
		for (int i = 0; i < _size; ++i) {
			_columnNames[i] = columnDefinitions.getName(i);
			_columnTypes[i] = columnDefinitions.getType(i).getName();
		}
	}

	public Map<String, Object> visitRow(Row row) {
		final Map<String, Object> result = new HashMap<String, Object>();
		for (int i = 0; i < _size; ++i) {
			result.put(_columnNames[i], visitColumn(i, row, _columnTypes[i]));
		}
		return result;
	}

	public Object visitColumn(int columnIndex, Row row, DataType.Name type) {
		switch (type) {
		case ASCII:
			return row.getString(columnIndex);
		case BIGINT:
			return row.getLong(columnIndex);
		case BLOB:
			return row.getBytes(columnIndex);
		case BOOLEAN:
			return row.getBool(columnIndex);
		case COUNTER:
			return row.getLong(columnIndex);
		case CUSTOM:
			return visitCustom(columnIndex, row);
		case DECIMAL:
			return row.getDecimal(columnIndex);
		case DOUBLE:
			return row.getDouble(columnIndex);
		case FLOAT:
			return row.getFloat(columnIndex);
		case INET:
			return row.getInet(columnIndex);
		case INT:
			return row.getInt(columnIndex);
		case LIST:
			return row.getList(columnIndex, Object.class);
		case MAP:
			return row.getMap(columnIndex, Object.class, Object.class);
		case SET:
			return row.getSet(columnIndex, Object.class);
		case TEXT:
			return row.getString(columnIndex);
		case TIMESTAMP:
			return row.getDate(columnIndex);
		case TIMEUUID:
			return row.getUUID(columnIndex);
		case UUID:
			return row.getUUID(columnIndex);
		case VARCHAR:
			return row.getString(columnIndex);
		case VARINT:
			return row.getVarint(columnIndex);
		default:
			throw new IllegalArgumentException("Unknowmn column type!");
		}
	}

	/**
	 * Visitor for custom data types.
	 * 
	 * @param columnIndex
	 * @param row
	 * @return null by default
	 */
	public Object visitCustom(int columnIndex, Row row) {
		return row.getBytesUnsafe(columnIndex);
	}
	
	public String[] getColumnNames() {
		return _columnNames;
	}
	
	public DataType.Name[] getColumnTypes() {
		return _columnTypes;
	}
}
