package ch.uzh.ifi.ddis.ifp.esper.cassandra;

/*
 * #%L
 * Cassandra for Esper
 * %%
 * Copyright (C) 2013 University of Zurich
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * #L%
 */

import java.util.Iterator;
import java.util.NoSuchElementException;

import com.datastax.driver.core.ResultSet;
import com.datastax.driver.core.Row;
import com.espertech.esper.client.EventBean;
import com.espertech.esper.client.EventBeanFactory;

/**
 * <p>
 * Generic implementation of an {@link java.util.Iterator} over an Cassandra
 * {@link ResultSet}.
 * </p>
 * 
 * @author Thomas Scharrenbach
 * @version 0.2.6
 * @since 0.2.6
 * 
 */
public class EsperResultSetIterator implements java.util.Iterator<EventBean> {

	private final ResultSet _resultSet;

	private final CassandraRowVisitor _rowVisitor;

	private final Iterator<Row> _it;

	private final EventBeanFactory _eventBeanFactory;

	public EsperResultSetIterator(ResultSet resultSet,
			EventBeanFactory eventBeanFactory) {
		_resultSet = resultSet;
		_it = _resultSet.iterator();
		_rowVisitor = new CassandraRowVisitor(_resultSet.getColumnDefinitions());
		_eventBeanFactory = eventBeanFactory;
	}

	@Override
	public boolean hasNext() {
		return _it.hasNext();
	}

	@Override
	public EventBean next() {
		if (!hasNext()) {
			throw new NoSuchElementException();
		} else {
			final Row cassandraRow = _it.next();
			return _eventBeanFactory.wrap(_rowVisitor.visitRow(cassandraRow));
		}
	}

	@Override
	public void remove() {
		throw new UnsupportedOperationException("Removal is not supported by "
				+ EsperResultSetIterator.class.getCanonicalName());
	}

}
